#!/usr/bin/python
# -*- coding: latin1 -*-
# $Id$
#
# Author:	Vasilis.Vlachoudis@cern.ch
# Date:	29-Mar-2012

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import os, os.path
import re
import sys

CLEAR     = "\033[2J"
BOLD      = "\033[1m"
UNDERLINE = "\033[4m"
REVERSE   = "\033[7m"
NORMAL    = "\033[m"

# MINIMUM VERSIONS
FLAIR_VERSION = "2.2-5"
FLUKA_VERSION = "2011.2c.5"

warnings = 0
def abort(w,title,hlp):
	global warnings

	print "%s%s %s %s%s"%("="*30,BOLD,w,NORMAL,"="*30)
	print "%s%s%s"%(BOLD,title,NORMAL)
	print hlp
	print "="*70
	if w=="ERROR":
		sys.exit(0)
	else:
		warnings += 1

# Check subprocess needed later
print "\nChecking python"
try:
	from subprocess import *
except:
	abort("ERROR",
		"python is old version and do not have the 'subprocess' module",
		"""
Install the latest version of python-2 for your system.
Recommended version: 2.7.+
Possible commands (as superuser):
Fedora, RedHast, Scientific Linux, CentOS, Suse:
	yum -y install python
Ubuntu:
	apt-get install python
		""")

# Check Python Version
if sys.version_info[0] >= 3 or \
   sys.version_info[0] < 2  or \
   sys.version_info[1] < 4:
	abort("ERROR",
		"flair only works with only with python V2 from 2.4+ (but not V3)",
		"""
Flair doesn't work with python V3.+.
Install the latest version of python-2 for your system.
Recommended version: 2.7.+
Possible commands (as superuser):
Fedora, RedHast, Scientific Linux, CentOS, Suse:
	yum -y install python
Ubuntu:
	apt-get install python

If multiple python versions are present in the system ensure that flair is
using the proper one by setting the environment variable PYTHON
		""")
print "\tVersion: %d.%d-%d ok"%(sys.version_info[0],sys.version_info[1],sys.version_info[2])

# Tkinter
print "\nChecking python tkinter module"
try:
	import Tkinter
except:
	abort("ERROR",
		"Tkinter library is missing",
		"""
Install the latest 'Tkinter' library compatible with your python
Possible commands (as superuser):
Fedora, RedHast, Scientific Linux, CentOS:
	yum -y install tkinter
Suse:
	yum -y install python-tk
Ubuntu, Debian:
	apt-get install python-tk
		""")
print "\tTkinter exists"

# Check Tkinter version
print "\nChecking tk library"
if Tkinter.TclVersion < 8.4:
	abort("WARNING",
		"Old Tkinter %g found"%(Tkinter.TclVersion),
		"""
Install the latest 'Tkinter' library compatible with your python
		""")
print "\tTk Version=", Tkinter.TclVersion,"ok"


# PIL
print "\nChecking python imaging"
try:
	import PIL.Image
	import PIL.ImageTk
except:
	abort("ERROR",
		"Python imaging library is missing",
		"""
Install the latest 'python imaging' and its tk interface compatible with your python
Possible commands (as superuser):
Fedora, RedHast, Scientific Linux, CentOS, Suse:
	yum -y install python-imaging-tk
Ubuntu, Debian:
	apt-get install python-imaging-tk
		""")
print "\tpython-imaging-tk installed"

# xterm
#print "\nChecking xterm"
#p = Popen("which --skip-alias xterm", shell=True, stdout=PIPE, close_fds=True)
#xterm = None
#while True:
#	line = p.stdout.readline()
#	if line=="": break
#	if line.find(" no xterm in ")<0:
#		xterm = line.strip()
#if xterm is None:
#	abort("ERROR",
#		"xterm program is not found",
#		"""
#Install the xterm program with
#Possible commands (as superuser):
#Fedora, RedHast, Scientific Linux, CentOS, Suse:
#	yum -y install xterm
#Ubuntu, Debian:
#	apt-get install xterm
#		""")
#else:
#	print "\txterm ok"

# Gnuplot
print "\nChecking gnuplot"
gnuplot = Popen("gnuplot", shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE)

gnuplot.stdin.write("show version\n")
gnuplot.stdin.write("quit\n")
if gnuplot.wait() != 0:
	abort("ERROR",
		"gnuplot not found. Message: %s"%(gnuplot.stderr.readline()),
		"""
Install the latest 'gnuplot'
Possible commands (as superuser):
Fedora, RedHast, Scientific Linux, CentOS, Suse:
	yum -y install gnuplot
Ubuntu, Debian:
	1st enable the universe source with:
		Go to "System Settings" -> "Software Sources"
		On the "Ubuntu Software" tab
		Check the "Community-maintained Open Source software (universe)"
	Then install gnuplot from the command line
		apt-get update
		apt-get install gnuplot-x11

Warning: gnuplot V4.4.3 has some problems with the color palette
		""")
else:
	i,o = gnuplot.communicate()
	for line in o.splitlines():
		m = re.match(r".*Verfluka-2011.2c-3.i686.rpmsion (\d+)\.(\d+) patchlevel \b(\S+)\b .*",o,re.M|re.S)
		if m:
			version = int(m.group(1))
			release = int(m.group(2))
			try:
				patch = int(m.group(3))
			except:
				patch = -1
			break
	else:
		version = 0
		release = -1
		patch   = -1
	gnuplot.stdin.close()

	if version == 0:
		print "Unable to find out the version of gnuplot"

	elif version < 4 or (version==4 and release<2):
		abort("ERROR",
			"gnuplot too old %d.%d.%d"%(version,release,patch),
			"""
Install the latest 'gnuplot'
Possible commands (as superuser):
Fedora, RedHast, Scientific Linux, CentOS, Suse:
	yum -y install gnuplot
Ubuntu, Debian:
	1st enable the universe source with:
		Go to "System Settings" -> "Software Sources"
		On the "Ubuntu Software" tab
		Check the "Community-maintained Open Source software (universe)"
	Then install gnuplot from the command line
		apt-get update
		apt-get install gnuplot-x11

Warning: gnuplot V4.4.3 has some problems with the color palette
			""")

	if version==4 and release==4 and patch==3:
		abort("WARNING",
			"gnuplot %d.%d.%d has a problem with the color palette"%(version,release,patch),
			"""
The problem exist only with 'x11' as terminal of gnuplot
Either, install a higher 'gnuplot' version 4.4.4+ or 4.4.2
Or, modify in flair "Tools->Preferences->Gnuplot"
the terminal to "wxt". But ensure that wxt is working
			""")
print "\tgnuplot %d.%d.%d ok"%(version,release,patch)

# Check system memory
print "\nChecking memory"
p = Popen("cat /proc/meminfo", shell=True, stdout=PIPE, close_fds=True)
ram = 0
swap = 0
while True:
	line = p.stdout.readline()
	if line=="": break
	s = line.split()
	if s[0] == "MemTotal:":
		ram = int(s[1]) # kB
	elif s[0] == "SwapTotal:":
		swap = int(s[1]) # kB

total = ram + swap

if ram == 0:
	abort("WARNING",
		"Unable to determine RAM",
		"Please ensure a minimum of 1 GB of RAM (not swap).")
elif ram < 1024*1024:
	abort("WARNING",
		"Not enough RAM %d kB to run FLUKA and flair "%(ram),
		"Please increase RAM to minimum 1GB.")
else:
	print "\tRam: %d kb ok"%(ram)

if swap == 0:
	abort("WARNING",
		"Unable to determine SWAP space",
		"Please ensure a minimum of 512 MB of SWAP space.")
elif swap < 512*1024:
	abort("WARNING",
		"Not enough SWAP %d kB to run FLUKA and flair "%(swap),
		"Please increase SWAP to minimum 512MB.")
else:
	print "\tSwap: %d kb ok"%(swap)

# Check CPU
print "\nChecking CPU"
ncpu = 0
cpufreq = 0

p = Popen("uname -m", shell=True, stdout=PIPE, close_fds=True)
while True:
	line = p.stdout.readline()
	if line=="": break
	arch = line.strip()

p = Popen("cat /proc/cpuinfo", shell=True, stdout=PIPE, close_fds=True)
while True:
	line = p.stdout.readline()
	if line=="": break
	s = line.split()
	if not s: continue
	if s[0] == "processor":
		ncpu = max(ncpu, int(s[2]))+1
	elif s[0] == "cpu" and s[1] == "MHz":
		cpufreq = float(s[3])

if ncpu == 0 or cpufreq == 0:
	abort("WARNING",
		"Unable to determine CPU type/speed",
		"Please ensure a minimum of 1.0 GHz of CPU speed..")
elif cpufreq < 1024:
	abort("WARNING",
		"Slow CPU speed %g MHz "%(cpufreq),
		"Please ensure you are not in PowerSaving mode.")
else:
	print "\tCPUs: %d ok"%(ncpu)
	print "\tClock: %g MHz ok"%(cpufreq)
	print "\tArch: %s"%(arch)

# Check x11 for 32 bit a high resolution
# Check CPU
print "\nChecking X11"
width = height = depth = 0
p = Popen("xdpyinfo", shell=True, stdout=PIPE, close_fds=True)
while True:
	line = p.stdout.readline()
	if line=="": break
	s = line.split()
	if not s: continue
	if s[0] == "dimensions:":
		if width==0:
			width, height = map(int,s[1].split("x"))
	elif s[0]=="depth" and s[1]=="of" and s[2]=="root":
		depth = int(s[4])

if width == 0 or depth == 0:
	abort("WARNING",
		"Unable to determine X11 dimensions and pixel depth",
		"""
Please ensure a minimum of 1280x768 with 24bit depth.
Install the xorg-x11-utils package to be able to determine the x11 dimensions
Possible commands (as superuser):
rpm based:
	yum -y install xorg-x11-utils
deb based:
	apt-get install xorg-x11-utils
		""")
else:
	if width < 1280 or height<768:
		abort("WARNING",
			"Screen dimensions too small %dx%d"%(width,height),
			"Please ensure a minimum of 1280x768 with 24bit depth.")
	if depth < 24:
		abort("ERROR",
			"Pixel depth too small %d bit"%(depth),
			"flair works only with minimum 24 bit depth.")
	print "\tX11 dimension: %dx%d ok"%(width,height)
	print "\tX11 depth: %d ok"%(depth)

# Check flair
print "\nChecking flair"
try:
	flairdir = "/usr/local/flair"
	os.stat(flairdir)
except:
	print "\t flair not in standard directory"
	flairdir = ""
	p = Popen("which --skip-alias flair", shell=True, stdout=PIPE, close_fds=True)
	while True:
		line = p.stdout.readline()
		if line=="": break
		if line.find(" no flair in ")<0:
			flairdir = os.path.dirname(line.strip())

	if not flairdir:
		abort("WARNING",
			"flair found",
			"""
Please install the latest version of flair. Or if it is already
installed please add flair to your PATH
Possible commands (as superuser):
Fedora, RedHast, Scientific Linux, CentOS, Suse:
	rpm -ivh --nodeps http://www.fluka.org/flair/flair-%s.noarch.rpm
Ubuntu:
	Download flair-%s_all.deb from the http://www.fluka.org/flair/download.html
	and install with
		dpkg -i flair-%s_all.deb
For more information please look in the flair readme file
			"""%(FLAIR_VERSION, FLAIR_VERSION, FLAIR_VERSION))
	while True:
		try:
			s = os.stat("%s/flair.py"%(flairdir))
			break
		except:
			print "\t%s/flair.py not found"%(flairdir)
			sys.stdout.write("Enter flair directory: ")
			flairdir = os.path.expandvars(os.path.expanduser(raw_input()))
			if flairdir=="":
				sys.exit(0)
print "\tflair directory: %s"%(flairdir)

# Check flair version
sys.path.append(flairdir)
sys.path.append("%s/lib"%(flairdir))
try:
	import tkFlair
except:
	abort("ERROR",
		"Cannot load flair from directory %s"%(flairdir),
		"Please verify flair installation")
print "\tflair version:",tkFlair.__version__

# Check geoviewer
print "\nChecking flair-geoviewer"
try:
	os.stat("%s/geoviewer.so"%(flairdir))
except:
	abort("ERROR",
		"Module: %s/geoviewer.so not found in flair directory"%(flairdir),
		"""
Please install the flair-geoviewer package,
rpm based:
	rpm -ivh flair-geoviewer-X.Y-Z.ARCH.rpm
deb based:
	dpkg -i flair-geoviewer_X.Y-Z_ARCH.deb
from source:
	follow the installation instructions in the flair README file
where: X.Y-Z is the latest flair version
	ARCH is the appropriate architecture for your machine
	If there is not available for your machine please try to
	compile and install from source (tgz)
		""")

try:
	import geoviewer

except:
	print >>sys.stderr,sys.exc_info()[1]
	abort("ERROR",
		"geoviewer module cannot be loaded",
		"""
Please install the CORRECT flair-geoviewer package for your LINUX architecture,
rpm based:
	rpm -ivh flair-geoviewer-X.Y-Z.ARCH.rpm
deb based:
	dpkg -i flair-geoviewer_X.Y-Z_ARCH.deb
from source (prefered):
	follow the instructions in the flair README file.
where: X.Y-Z is the latest flair version
	ARCH is the appropriate architecture for your machine
	If there is not available for your machine please try to
	compile and install from source (tgz)
		""")

if tkFlair.__version__ < FLAIR_VERSION:
	abort("WARNING",
		"flair version %s found, expected minimum %s"%(tkFlair.__version__, FLAIR_VERSION),
		"""
Please install the latest flair & flair-geoviewer version.
Website: http://www.fluka.org/flair/download.html
		""")
else:
	print "\tflair version ok (same or higher than %s)"%(FLAIR_VERSION)

print "\tflair-geoviewer version",geoviewer.__version__
if geoviewer.__version__ != tkFlair.__version__:
	abort("ERROR",
		"Version mismatch flair <> flair-geoviewer",
		"Please install the same version of flair-geoviewer and flair package")
else:
	print "\tflair and geoviewer version match"


# Check g77
print "\nChecking g77"

f = open("test.f","w")
print >>f,"         program test"
print >>f,"         write(*,*) 'Hello'"
print >>f,"         stop"
print >>f,"         end"
f.close()

g77 = None
p = Popen("g77 -v", shell=True, stderr=PIPE, close_fds=True)
while True:
	line = p.stderr.readline()
	if line=="": break
	if line.find("version")>=0:
		g77 = line.split()[2]
		break

if g77 is None:
	abort("WARNING",
		"g77 fortran not found",
		"""
Please install the g77 or the gfortran 4.6+ (ONLY for 86bit architecture)
Fedora, RedHast, Scientific Linux, CentOS, Suse:
	yum -y install compat-gcc-34-g77
Ubuntu:
	g77 has been withdrawn from the standard packages.
	Please install the x86_64 bit version of gfortran
		""")

else:
	print "\tg77 version",g77

	# Check if it works
	err = False
	p = Popen("g77 test.f", shell=True, stderr=PIPE, close_fds=True)
	while True:
		line = p.stderr.readline()
		if line=="": break
		if line.strip() <> "":
			print line[:-1]
			err = True

	if err:
		abort("WARNING",
			"g77 fortran not working properly",
			"Please install the g77 or the gfortran 4.6+")

	else:
		p = Popen("./a.out", shell=True, stdout=PIPE, close_fds=True)
		while True:
			line = p.stdout.readline()
			if line=="": break
			if line.strip() == "Hello":
				break
			else:
				err = True

		if err:
			abort("WARNING",
				"g77 fortran is not compiling ok",
				"Please install the g77 or the gfortran 4.6+")
		else:
			print "\tg77 function properly"

		try: os.remove("a.out")
		except: pass

# Check for gfortran
gfortran = None
print "\nChecking gfortran"
if arch != "x86_64":
	print "\tNot 64bit architecture skipping gfortran check"
else:
	p = Popen("gfortran -dumpversion", shell=True, stdout=PIPE, close_fds=True)
	while True:
		line = p.stdout.readline()
		if line=="": break
		gfortran = line.strip()

	if gfortran is None:
		abort("WARNING",
			"g77 fortran not found",
			"Please install the g77 or the gfortran 4.6+")
	else:
		print "\tgfortran version",gfortran

		if gfortran < "4.5.0":
			abort("WARNING",
				"gfortran version 4.5 or higher is required",
				"Recommended version 4.6.0")

		# Check if it works
		err = False
		p = Popen("gfortran test.f", shell=True, stderr=PIPE, close_fds=True)
		while True:
			line = p.stderr.readline()
			if line=="": break
			if line.strip() <> "":
				err = True

		if err:
			abort("WARNING",
				"gfortran not working properly",
				"Please install the latest gfortran 4.6+")
		else:
			p = Popen("./a.out", shell=True, stdout=PIPE, close_fds=True)
			while True:
				line = p.stdout.readline()
				if line=="": break
				if line.strip() == "Hello":
					break
				else:
					err = True

			if err:
				abort("WARNING",
					"gfortran is not compiling ok",
					"Please install the latest gfortran 4.6+")
			else:
				print "\tgfortran function properly"

try:
	os.remove("test.f")
	os.remove("a.out")
except:
	pass

# Check FLUPRO
print "\nChecking FLUKA"
flupro = os.getenv("FLUPRO")
if flupro is None:
	abort("ERROR",
		"FLUPRO variable is not set",
		"Please set the correct FLUPRO variable")

fluka = None
try:
	f = open("%s/Version.tag"%(flupro),"r")
	fluka = f.readline().split()[-1]
	f.close()
	print "\tFLUPRO=%s ok"%(flupro)
except:
	abort("ERROR",
		"fluka Version.tag file not found in FLUPRO=%s directory"%(flupro),
		"Please install correctly FLUKA and set the FLUPRO environment variable")

if fluka < FLUKA_VERSION:
	abort("WARNING",
		"fluka version %s found, expected minimum %s"%(fluka, FLUKA_VERSION),
		"Please install correctly FLUKA and set the FLUPRO environment variable")
else:
	print "\tFLUKA Version=",fluka

# Check flukahp (output returned)
try:
	os.stat("%s/flukahp"%(flupro))
	print "\t%s/flukahp exists"%(flupro)
except:
	abort("ERROR",
		"Fluka executable %s/flukahp not found"%(flupro),
		"Please ensure you have correctly compiled fluka using 'make'")

# Check utilities usbsuw, gplevbin
try:
	os.stat("%s/flutil/usbsuw"%(flupro))
	os.stat("%s/flutil/ustsuw"%(flupro))
	os.stat("%s/flutil/gplevbin"%(flupro))
	print "\t%s/flutil utilities exists ok"%(flupro)
except:
	abort("ERROR",
		"Fluka utilities %s/flutil/* are not compiled properly"%(flupro),
		"Please ensure you have correctly compiled fluka using 'make'")

# Check if we can compile flukahp
p = Popen("%s/flutil/lfluka -m fluka"%(flupro), shell=True, stdout=PIPE, stderr=PIPE, close_fds=True)
err = None
while True:
	line = p.stderr.readline()
	if line=="": break
	if line.find("ld returned")>=0:
		print line
		err = True

try:
	os.stat("flukahp")
except:
	err = True

if err:
	abort("ERROR",
		"$FLUPRO/flutil/lfluka -m fluka   doesn't compile/link properly",
		"""
		Please ensure you have the correct version of fluka with the appropriate compiler
		For g77, please ensure that the glibc-devel package (for i686 architecture) is
		available.
		""")
else:
	print "\t$FLUPRO/flutil/lfluka -m fluka   compiles ok"

# Check if flukahp runs!
fluka = False
f = open("test.inp","w")
print >>f,"STOP"
f.close()
p = Popen("./flukahp < test.inp", shell=True,  stdout=PIPE, close_fds=True)
while True:
	line = p.stdout.readline()
	if line=="": break
	if line.strip() == "FLUKA User license":
		fluka = True
try:
	os.remove("test.inp")
	os.remove("fort.11")
	os.remove(".timer.out")
	os.remove("fort.77")
except: pass
if not fluka:
	abort("ERROR",
		"Fluka doesn't run properly",
		"Please ensure you have correctly compiled fluka using 'make'")
else:
	print "\tflukahp runs ok"

try:
	os.remove("flukahp.map")
	os.remove("flukahp")
except:
	pass

# Epilogue
print
print
print "Congratulations all tests passed"
print
if warnings:
	print warnings,"warnings found"

print
print "However this doesn't mean that everything is functioning properly"
print "Start flair, verify that the output doesn't contain any error or warning message"
print "Create a small input, run and plot the results :)"
